<?php

/**
 * @file
 * Cloudwords queue management page.
 */

/**
 * Custom submit callback for marking a translatable out of date.
 */
function cloudwords_outdated_form_submit($form, &$form_state) {
  if (!empty($form_state['values']['cloudwords_outdated'])) {
    foreach ($form_state['cloudwords_translatables'] as $translatable) {
      if ($translatable->translation_status == 1) {
        $translatable->translation_status = 2;
        $translatable->save();
      }
    }
  }
}


/**
 * Provides the overview page for Cloudwords.
 */
function cloudwords_project_overview_form($form, &$form_state) {
  $projects = cloudwords_get_api_client()->get_open_projects();

  $existing_ids = db_query("SELECT pid FROM {cloudwords_project}")->fetchCol();

  $rows = array();

  foreach ($projects as $project) {
    if (in_array($project->getId(), $existing_ids)) {
      $params = $project->getParams();

      // Gather target languages. Wrap every 4 languages.
      $langs = array();
      $key = 0;
      foreach ($params['targetLanguages'] as $delta => $lang) {
        if ($delta % 4 === 0) {
          $key++;
        }
        $langs[$key][] =  $lang['display'];
      }
      foreach ($langs as $delta => $lang) {
        $langs[$delta] = implode(', ', $langs[$delta]);
      }
      $targetLanguage = implode('<br />', $langs);

      $row = array(
        l($params['name'], 'admin/structure/cloudwords/project/' . $params['id']),
        $params['status']['display'],
        isset($params['sourceLanguage']) ? $params['sourceLanguage']['display'] : '',
        $targetLanguage,
      );

      $rows[] = $row;
    }
  }

  $header = array(
    t('Name'),
    t('Status'),
    t('Source language'),
    t('Target languages'),
  );

  $form['table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#attributes' => array('id' => 'cloudwords-project-table'),
    '#empty' => t('No projects available.'),
  );

  return $form;
}

/**
 * Provides the overview page for Cloudwords.
 */
function cloudwords_project_closed_overview_form($form, &$form_state) {

// Select table
  $projects = db_select('cloudwords_project', 'cp')
    ->fields('cp', array('pid', 'name', 'status'))
    ->condition('status', cloudwords_project_closed_statuses())
    // ->extend('TableSort')
    ->extend('PagerDefault')
    ->limit(50)
    ->execute()
    ->fetchAll();

  // Execute query
  // $projects = $query->execute();

  // $projects = db_query("SELECT pid, name, status FROM {cloudwords_project} WHERE status IN (:statuses)", array(':statuses' => cloudwords_project_closed_statuses()));


  $rows = array();

  foreach ($projects as $project) {
    $row = array();
    $row[] = l($project->name, 'admin/structure/cloudwords/project/' . $project->pid);
    switch ($project->status) {
      case 'cancelled':
        $row[] = t('Cancelled');
        break;

      case 'project_closed':
        $row[] = t('Closed');
        break;
    }
    $rows[] = $row;
  }

  $header = array(
    t('Name'),
    t('Status'),
  );

  $form['table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#attributes' => array('id' => 'cloudwords-project-table'),
    '#empty' => t('No closed projects.'),
  );

  $form['pager'] = array(
    '#theme' => 'pager',
  );

  return $form;
}

/**
 * The project detail page.
 */
function cloudwords_project_form($form, &$form_state, CloudwordsDrupalProject $project) {
  $form_state['cloudwords_project'] = $project;

  $action_statuses = array(
    'configured_project_name',
    'configured_project_details',
    'uploaded_source_materials',
    'configured_bid_options',
    'waiting_for_bid_selection',
    'bid_selection_expired',
  );

  if (in_array($project->getStatus()->getCode(), $action_statuses)) {
    drupal_set_message(t('Please finish <a href="!href" target="_blank">creating your project in Cloudwords</a>.', array('!href' => _cloudwords_ui_url() . '/cust.htm#project/' . $project->getId())), 'warning');
  }

  $project_info = array(
    'pid' => $project->getId(),
    'name' => $project->getName(),
    'status' => $project->getStatus()->getCode(),
  );
  drupal_write_record('cloudwords_project', $project_info, array('pid'));

  $client = cloudwords_get_api_client();


  // Build reference material table.
  try {
    $references = $client->get_project_references($project->getId());
  }
  catch (CloudwordsApiException $e) {
    $references = array();
  }
  $reference_rows = array();
  foreach ($references as $reference) {
    $ops = array(
      l(t('Replace'), 'admin/structure/cloudwords/file/' . $project->getId() . '/' . $reference->getId() . '/replace'),
      // l(t('Delete'), 'admin/structure/cloudwords/file/' . $project->getId() . '/' . $reference->getId() . '/delete'),
      l(t('Download'), 'admin/structure/cloudwords/file/' . $project->getId() . '/' . $reference->getId() . '/download'),
    );

    $row = array(
      $reference->getFileName(),
      _cloudwords_format_display_date($reference->getCreatedDate()),
      implode(' | ', $ops),
    );
    $reference_rows[] = $row;
  }

  // $form['project_info'] = array(
  //   '#type' => 'fieldset',
  // );
  // $form['project_info']['source'] = array(
  //   '#type' => 'container',
  //   '#title' => t('Source language'),
  //   '#value' => 'asdf',
  // );

  $form['project_details'] = array(
    '#type' => 'fieldset',
    '#title' => t('Project Details'),
    '#tree' => TRUE,
  );
  $form['project_details']['metadata']['source_language'] = array(
    '#markup' => t('Source language') . ': ' . $project->getSourceLanguage()->getDisplay() . '<br>',
  );
  $form['project_details']['metadata']['status'] = array(
    '#markup' => t('Status') . ': ' . $project->getStatus()->getDisplay() . '<br>',
  );
  $form['project_details']['metadata']['description'] = array(
    '#markup' => t('Description') . ': ' . $project->getDescription() . '<br>',
  );
  $form['project_details']['metadata']['notes'] = array(
    '#markup' => t('Notes') . ': ' . $project->getNotes() . '<br>',
  );
  $form['project_details']['metadata']['delivery_due_date'] = array(
    '#markup' => t('Delivery due date') . ': ' . _cloudwords_format_display_date($project->getDeliveryDueDate()) . '<br>',
  );

  $options = array(
    'attributes' => array(
      'target' => '_blank',
    ),
  );
  $form['project_details']['metadata']['view'] = array(
    '#markup' => l(t('View Project in Cloudwords'), _cloudwords_ui_url() . '/cust.htm#project/' . $project->getId(), $options),
  );

  // $reference_rows[] = array(array('data' => $form['upload']), 'attributes' => array('colspan' => 3));

  // Build translation files table.
  $files = array();
  try {
    $files = $client->get_project_translated_files($project->getId());
  }
  catch (CloudwordsApiException $e) {}

  $rows = array();
  foreach ($files as $file) {
    $lang_path = 'admin/structure/cloudwords/project/' . $project->getId() . '/' . $file->getLang()->getLanguageCode();
    $status_code = $file->getStatus()->getCode();

    $operations = array();
    if ($status_code != 'not_delivered') {
      $operations[] = l(t('Import'), $lang_path . '/import');
    }

    $language_status = $project->getLanguageImportStatus($file->getLang());

    $query = db_select('cloudwords_content', 'cc');
    $query->addJoin('INNER', 'cloudwords_translatable', 'ct', 'ct.ctid = cc.ctid');
    $query->condition('ct.language', $file->getLang()->getLanguageCode());
    $query->addExpression('COUNT(cc.ctid)', 'ncount');
    $total = $query->execute()->fetchField();
    $failed = $query->condition('cc.status', 3)->execute()->fetchField();

    if ($status_code == 'approved' && $language_status != CLOUDWORDS_LANGUAGE_FAILED) {
      $language_status = t('Imported');
    }
    elseif ($language_status == 1 && $status_code == 'delivered') {
      $language_status = t('Imported');
      $operations[] = l(t('Approve'), 'admin/structure/cloudwords/project/' . $project->getId() . '/' . $file->getLang()->getLanguageCode() . '/approve');
    }
    else if ($language_status == CLOUDWORDS_LANGUAGE_APPROVED) {
      $language_status = t('Imported');
    }
    else if ($language_status == CLOUDWORDS_LANGUAGE_FAILED) {
      $language_status = '<span class="marker">' . t('Failed') . '</span>';
    }
    else {
      $language_status = t('Not imported');
    }

    if ($failed) {
      $language_status = '<span class="marker">' . "$failed/$total failed" . '</span>';
    }

    $row = array(
      l($file->getLang()->getDisplay(), $lang_path),
      $file->getStatus()->getDisplay(),
      $language_status,
      implode(' | ', $operations),
    );
    $rows[] = $row;
  }
  $header = array(
    t('Name'),
    t('Status'),
    t('Import status'),
    t('Operations'),
  );

  $form['language_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Languages'),
    '#tree' => FALSE,
  );

  $form['language_wrapper']['table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#attributes' => array('id' => 'cloudwords-project-table'),
    '#empty' => t('No languages available.'),
  );

  $form['reference_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Project Reference Materials (Optional)'),
    '#tree' => TRUE,
    '#collapsible' => TRUE,
    '#collapsed' => empty($reference_rows),
  );

  $form['reference_wrapper']['reference_table'] = array(
    '#theme' => 'table',
    '#header' => array(t('File name'), t('Date'), t('Operations')),
    '#rows' => $reference_rows,
    '#attributes' => array('id' => 'cloudwords-project-table'),
    '#empty' => t('No reference material available.'),
  );

  $form['reference_wrapper']['upload'] = array(
    '#type' => 'container',
    '#title' => t('Project reference material'),
    '#attributes' => array('class' => array('container-inline')),
    '#tree' => FALSE,
  );

  $form['reference_wrapper']['upload']['reference'] = array(
    '#type' => 'file',
    // '#title' => t('Project reference materials'),
    // '#description' => t('Upload additional reference materials.'),
    // '#theme' => 'feeds_upload',
    '#file_info' => NULL,
    '#size' => 10,
  );

  $form['reference_wrapper']['upload']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Upload'),
  );

  return $form;
}

/**
 * Validate callback for cloudwords_project_form().
 */
function cloudwords_project_form_validate($form, &$form_state) {
  $upload_dir = 'private://cloudwords/reference_material';

  if (!file_prepare_directory($upload_dir, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
    form_set_error('upload][reference', t('Unable to create the upload directory.'));
  }

  if (!($file = file_save_upload('reference', array('file_validate_extensions' => array('zip')), $upload_dir))) {
    // form_set_error('upload][reference', t('Please upload a zip file.'));
  }
  else {
    $form_state['reference_material'] = $file;
  }
}

/**
 * Submit callback for cloudwords_project_form().
 */
function cloudwords_project_form_submit($form, &$form_state) {
  $project = $form_state['cloudwords_project'];

  if (!empty($form_state['reference_material'])) {
    cloudwords_get_api_client()->upload_project_reference($project->getId(), drupal_realpath($form_state['reference_material']->uri));
  }
}

/**
 * The language detail page for a project.
 */
function cloudwords_project_language_form($form, &$form_state, CloudwordsDrupalProject $project, CloudwordsLanguage $language) {
  $rows = array();

  $header_row = array(
    array('data' => 'Source name', 'field' => 'ct.label'),
    array('data' => 'Group', 'field' => 'ct.textgroup'),
    array('data' => 'type', 'field' => 'ct.type'),
    array('data' => 'Translation status', 'field' => 'ct.translation_status'),
    array('data' => 'Import status', 'field' => 'cc.status'),
  );

  $query = db_select('cloudwords_translatable', 'ct')
    ->fields('ct')
    ->condition('ct.language', $language->getLanguageCode());

    $query->addJoin('INNER', 'cloudwords_content', 'cc', 'ct.ctid = cc.ctid');
    $query->fields('cc', array('status'));
    $query->condition('cc.pid', $project->getId());

    $query
      ->extend('TableSort')
      ->orderByHeader($header_row)
      ->extend('PagerDefault')
      ->limit(5);

  $results = $query->execute();

  foreach ($results as $result) {
    $translatable = cloudwords_translatable_load($result->ctid);
    $row = array();

    $row['label'] = $translatable->editLink();
    $row['group'] = $translatable->textGroupLabel();
    $row['type'] = $translatable->typeLabel();
    $row['translation_status'] = $translatable->translationStatusLabel();

    switch ($result->status) {
      case CLOUDWORDS_LANGUAGE_NOT_IMPORTED:
        $status = t('Not imported');
        break;

      case CLOUDWORDS_LANGUAGE_IMPORTED:
        $status = t('Imported');
        break;

      case CLOUDWORDS_LANGUAGE_FAILED:
        $status = '<span class="marker">' . t('Import failed') . '</span>';
        break;
    }

    $row['status'] =  $status;
    $rows[$translatable->ctid] = $row;
  }

  $form['table'] = array(
    '#theme' => 'table',
    '#header' => $header_row,
    '#rows' => $rows,
    '#attributes' => array('id' => 'cloudwords-project-table'),
    '#empty' => t('No content available.'),
  );

  $form['pager'] = array(
    '#theme' => 'pager',
  );


  return $form;
}

/**
 * Confirmation form for importing a delived language.
 */
function cloudwords_project_language_import_form($form, &$form_state, CloudwordsDrupalProject $project, CloudwordsLanguage $language) {
  $form_state['cloudwords_project'] = $project;
  $form_state['cloudwords_language'] = $language;

  $question = t('Are you sure you want to import content for %lang?', array('%lang' => $language->getDisplay()));
  $path = 'admin/structure/cloudwords/project/' . $project->getId();

  $language_status = db_query("SELECT status FROM {cloudwords_project_language} WHERE pid = :pid AND language = :lang",
    array(':pid' => $project->getId(), ':lang' => $language->getLanguageCode())
  )->fetchField();
  $description = NULL;
  if ($language_status == 2) {
    $description = t('You have already imported content from this project and language. If you import again it will overwrite any existing content for this language.');
  }

  return confirm_form($form, $question, $path, $description);
}

/**
 * Submit callback for cloudwords_project_language_import_form().
 */
function cloudwords_project_language_import_form_submit($form, &$form_state) {
  $project = $form_state['cloudwords_project'];
  $language = $form_state['cloudwords_language'];

  // Download file and save as a temporary file.
  $file = cloudwords_get_api_client()->download_translated_file($project->getId(), $language->getLanguageCode());
  $temp = tempnam(file_directory_temp(), 'cloudwords-');
  file_put_contents($temp, $file);

  // Read zip file contents.
  $zip = new ArchiverZip($temp);
  $file_names = $zip->listContents();
  $archive = $zip->getArchive();

  if ($file_names) {
    $batch = array(
      'title' => t('Importing files ...'),
      'operations' => array(),
      'init_message' => t('Loading files'),
      'progress_message' => t('Processed @current out of @total files.'),
      'error_message' => t('An error occurred during processing.'),
      'finished' => 'cloudwords_import_batch_finished',
      'progressive' => FALSE,
      'file' => drupal_get_path('module', 'cloudwords') . '/cloudwords.pages.inc',
    );

    foreach ($file_names as $file_name) {
      if ($archive->statName($file_name, ZIPARCHIVE::FL_NODIR)) {
        $batch['operations'][] = array('cloudwords_import_batch', array($temp, $file_name, $project, $language));
      }
    }

    batch_set($batch);
  }
}

/**
 * Extracts one xml file from a zip file and imports it.
 */
function cloudwords_import_batch($zip_file, $file_name, $project, $language, &$context) {
  if (!isset($context['results']['processed'])) {
    $context['results']['processed'] = 0;
    $context['results']['zip_file'] = $zip_file;
    $context['results']['project'] = $project;
    $context['results']['language'] = $language;
    $context['results']['successful'] = 0;
    $context['results']['failed'] = 0;
  }

  $serializer = new CloudwordsFileformatXLIFF();
  $file = 'zip://' . $zip_file . '#' . $file_name;

  if ($serializer->validateImport($project, $language, $file)) {

    $valid_ctids = $project->getCtids($language->getLanguageCode());
    $imported = $serializer->import($file);

    $to_be_imported = array_intersect($valid_ctids, array_keys($imported));

    $translatables = cloudwords_translatable_load_multiple($to_be_imported);

    foreach ($imported as $ctid => $data) {

      if (!in_array($ctid, $to_be_imported)) {
        $context['results']['success'] = FALSE;
        $context['results']['failed']++;
        drupal_set_message(t('%file contains an invalid id: %id', array('%file' => $file, '%id' => $ctid)), 'error');
      }
      else {
        $translatable = $translatables[$ctid];
        try {
          $translatable->saveData($data);
          $translatable->translation_status = 1;
          $translatable->save();
          $context['results']['successful']++;
          db_merge('cloudwords_content')
            ->key(array(
              'pid' => $project->getId(),
              'ctid' => $translatable->ctid,
            ))
            ->fields(array(
              'status' => CLOUDWORDS_LANGUAGE_IMPORTED,
            ))
            ->execute();
        }
        catch (Exception $e) {
          drupal_set_message(t($e->getMessage()), 'error');
          $context['results']['failed']++;
          db_merge('cloudwords_content')
            ->key(array(
              'pid' => $project->getId(),
              'ctid' => $translatable->ctid,
            ))
            ->fields(array(
              'status' => CLOUDWORDS_LANGUAGE_FAILED,
            ))
            ->execute();
        }
      }
      $context['results']['processed']++;
      $context['message'] = t('Importing %label (%type).', array('%label' => $translatable->label, '%type' => $translatable->typeLabel()));
    }
  }
  else {
    drupal_set_message(t('%file failed to validate.', array('%file' => $file)), 'error');
    $context['results']['success'] = FALSE;
    $project->setLanguageImportStatus($language, CLOUDWORDS_LANGUAGE_FAILED);
  }
}

/**
 * Finished callback for batch importing.
 *
 * @todo Cleanup after error.
 */
function cloudwords_import_batch_finished($success, $results, $operations, $time) {
  $results += array('success' => TRUE);

  if ($success) {
    if ($results['successful']) {
      $message = format_plural($results['successful'],
        'Saved %count item.',
        'Saved %count items.',
        array('%count' => $results['successful'])
      );
      drupal_set_message($message);
    }

    if ($results['failed']) {
      $message = format_plural($results['failed'],
        'Failed saving %count item.',
        'Failed saving %count items.',
        array('%count' => $results['failed'])
      );
      drupal_set_message($message, 'error');
    }
  }

  // This means something blew up. bail.
  else {
    return;
  }

  if ($results['success']) {
    $results['project']->setLanguageImportStatus($results['language'], CLOUDWORDS_LANGUAGE_IMPORTED);
  }
  else {
    $results['project']->setLanguageImportStatus($results['language'], CLOUDWORDS_LANGUAGE_FAILED);
  }

  // Delete the temporary zip file.
  if (!empty($results['zip_file'])) {
    file_unmanaged_delete($results['zip_file']);
  }

  drupal_goto('admin/structure/cloudwords/project/' . $results['project']->getId());
}

function cloudwords_project_language_approve_form($form, &$form_state, CloudwordsDrupalProject $project, CloudwordsLanguage $language) {
  $form_state['cloudwords_project'] = $project;
  $form_state['cloudwords_language'] = $language;

  $question = t('Are you sure you want to approve %lang?', array('%lang' => $language->getDisplay()));
  $path = 'admin/structure/cloudwords/project/' . $project->getId();
  $description = t('Approving a language tells your translation vendor the content has been accepted and does not need further work. This action cannot be undone.');
  return confirm_form($form, $question, $path, $description);
}

function cloudwords_project_language_approve_form_submit($form, &$form_state) {
  $project = $form_state['cloudwords_project'];
  $language = $form_state['cloudwords_language'];

  cloudwords_get_api_client()->approve_project_language($project->getId(), $language->getLanguageCode());

  $project->setLanguageImportStatus($language, CLOUDWORDS_LANGUAGE_APPROVED);

  $langcode = $language->getLanguageCode();
  $ctids = db_query("SELECT cc.ctid FROM {cloudwords_content} cc
                     INNER JOIN {cloudwords_translatable} ct
                     WHERE cc.pid = :pid AND ct.language = :lang AND cc.ctid = ct.ctid", array(
    ':pid' => $project->getId(),
    ':lang' => $langcode,
  ))->fetchCol();

  db_update('cloudwords_translatable')
    ->condition('language', $langcode)
    ->condition('ctid', $ctids)
    ->fields(array(
      'status' => CLOUDWORDS_QUEUE_NOT_IN_QUEUE,
    ))
    ->execute();

  drupal_set_message(t('%name was successfully approved.', array('%name' => $project->getName())));

  $form_state['redirect'] = 'admin/structure/cloudwords/project/' . $project->getId();
}

/**
 * Replaces an existing reference file with a new one.
 */
function cloudwords_file_replace_form($form, &$form_state, CloudwordsDrupalProject $project, CloudwordsFile $file) {
  drupal_set_title(t('Replace %file', array('%file' => $file->getFilename())), PASS_THROUGH);
  $form_state['cloudwords_project'] = $project;
  $form_state['cloudwords_file'] = $file;

  $form['reference'] = array(
    '#type' => 'file',
    '#title' => t('Project reference materials'),
    '#description' => t('Upload additional reference materials as a zip file.'),
    // '#theme' => 'feeds_upload',
    '#file_info' => NULL,
    '#size' => 10,
  );

  $form['actions'] = array('#type' => 'actions');

  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Upload'),
  );

  $form['actions']['cancel'] = array(
    '#markup' => l(t('Cancel'), 'admin/structure/cloudwords/project/' . $project->getId()),
    '#weight' => '1000',
  );

  return $form;
}

/**
 * Validate callback for cloudwords_file_replace_form().
 */
function cloudwords_file_replace_form_validate($form, &$form_state) {
  $upload_dir = 'private://cloudwords/reference_material';

  if (!file_prepare_directory($upload_dir, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
    form_set_error('upload][reference', t('Unable to create the upload directory.'));
  }

  if (!($file = file_save_upload('reference', array('file_validate_extensions' => array('zip')), $upload_dir))) {
    // form_set_error('upload][reference', t('Please upload a zip file.'));
  }
  else {
    $form_state['reference_material'] = $file;
  }
}

/**
 * Submit callback for cloudwords_file_replace_form().
 */
function cloudwords_file_replace_form_submit($form, &$form_state) {
  if (!empty($form_state['reference_material'])) {
    $project = $form_state['cloudwords_project'];
    $file = $form_state['cloudwords_file'];
    $path = drupal_realpath($form_state['reference_material']->uri);
    $return = cloudwords_get_api_client()->update_project_reference($project->getId(), $file->getId(), $path);
    drupal_set_message(t('File %old was replaced with %new.', array('%old' => $file->getFilename(), '%new' => $return->getFilename())));
    $form_state['redirect'] = 'admin/structure/cloudwords/project/' . $project->getId();
  }
}

/**
 * Page callback that serves a file downloaded from Cloudwords.
 */
function cloudwords_file_download(CloudwordsDrupalProject $project, CloudwordsFile $file) {
  $filename = $file->getFilename();
  $filedata = cloudwords_get_api_client()->download_reference_file($project->getId(), $file->getId());
  header('Content-Disposition: attachment; filename=' . basename($filename));
  header('Content-Type: application/force-download');
  header('Content-Type: application/octet-stream');
  header('Content-Type: application/download');
  header('Content-Description: File Transfer');
  header('Content-Length: ' . strlen($filedata));
  echo $filedata;
  die();
}

function _cloudwords_format_display_date($date) {
  try {
    $datetime = new DateTime($date);
    $date = format_date($datetime->getTimestamp(), 'custom', 'm/d/Y', 'UTC');
  }
  catch (Exception $exc) {
    $date = '';
  }

  return $date;
}
