<?php

/**
 * @file
 *
 * Contains CloudwordsDrupalProject.
 */

class CloudwordsDrupalProject extends CloudwordsProject {

  /**
   * Returns a list of translatable ids for this project.
   *
   * @param $string $langcode
   *   (Optional) A specific language code to filter on.
   *
   * @return array
   *   A list of ctids.
   */
  public function getCtids($lang_code = NULL) {
    $query = db_select('cloudwords_content', 'cc')
      ->fields('cc', array('ctid'))
      ->condition('cc.pid', $this->getId());

    if ($lang_code) {
      $query->addJoin('INNER', 'cloudwords_translatable', 'ct', 'ct.ctid = cc.ctid');
      $query->condition('ct.language', $lang_code);
    }

    return $query->execute()->fetchCol();
  }

  /**
   * Returns a list of translatables
   *
   * @param string $langcode
   *   (Optional) A specific language code to filter on.
   *
   * @return array
   *   A list of CloudwordsTransltable objects.
   */
  public function getTranslatables($lang_code = NULL) {
    $ctids = $this->getCtids($lang_code);
    return cloudwords_translatable_load_multiple($ctids);
  }

  /**
   * Gets the import status for a language.
   *
   * @param CloudwordsLanguage $language
   *   A Cloudwords language object.
   */
  public function getLanguageImportStatus(CloudwordsLanguage $language) {
    $args = array(
      ':pid' => $this->getId(),
      ':lang' => $language->getLanguageCode(),
    );

    return db_query("SELECT status FROM {cloudwords_project_language} WHERE pid = :pid AND language = :lang", $args)->fetchField();
  }

  /**
   * Updates the import status for a language.
   *
   * @param CloudwordsLanguage $language
   *   A Cloudwords language object.
   * @param int $code
   *   The status code for the import.
   */
  public function setLanguageImportStatus(CloudwordsLanguage $language, $code) {

    db_merge('cloudwords_project_language')
      ->key(array(
        'pid' => $this->getId(),
        'language' => $language->getLanguageCode(),
      ))
      ->fields(array(
        'status' => $code,
      ))
      ->execute();
  }

}
